from pydub import AudioSegment

from bragir.tracing.logger import logger


def combine(audio_segments: list[AudioSegment]) -> AudioSegment | None:
    if not audio_segments:
        return None

    if len(audio_segments) == 1:
        return audio_segments[0]

    combined_no_crossfade: AudioSegment = audio_segments[0]

    for segment in audio_segments[1:]:
        # Can't type check this because of pydub's operator overloading
        combined_no_crossfade += segment

    assert isinstance(combined_no_crossfade, AudioSegment)

    return combined_no_crossfade


def group_audio_segments(
    audio_segments: list[AudioSegment], duration_limit_seconds: float
) -> list[AudioSegment]:
    logger.info(f"Grouping {len(audio_segments)} audio segments")

    if not audio_segments:
        return []

    total_duration = sum(obj.duration_seconds for obj in audio_segments)

    if total_duration < duration_limit_seconds:
        combined_audio_segment = combine(audio_segments)

        if combined_audio_segment is None:
            return []

        return [combined_audio_segment]

    grouped_audio_segments: list[AudioSegment] = []
    current_group: list[AudioSegment] = []
    total_duration = 0.0

    for obj in audio_segments:
        if total_duration + obj.duration_seconds > duration_limit_seconds:
            combined = combine(current_group)
            if combined:  # Ensuring that None is not appended
                grouped_audio_segments.append(combined)
            current_group = []
            total_duration = 0.0

        current_group.append(obj)
        total_duration += obj.duration_seconds

    # Handle the last group if not empty
    combined = combine(current_group)

    if combined:
        grouped_audio_segments.append(combined)

    logger.info(f"Grouped into {len(grouped_audio_segments)} audio segments")

    return grouped_audio_segments



from enum import StrEnum, auto
from typing import Any, Type

BCP47_TAGS = {
    "PORTUGUESE": "pt",
    "SPANISH": "es",
    "FRENCH": "fr",
    "POLISH": "pl",
    "GERMAN": "de",
    "ITALIAN": "it",
    "DUTCH": "nl",
    "SWEDISH": "sv",
    "DANISH": "da",
    "NORWEGIAN": "no",
    "FINNISH": "fi",
    "TURKISH": "tr",
}


class Languages(StrEnum):
    PORTUGUESE = auto()
    SPANISH = auto()
    FRENCH = auto()
    POLISH = auto()
    GERMAN = auto()
    ITALIAN = auto()
    DUTCH = auto()
    SWEDISH = auto()
    DANISH = auto()
    NORWEGIAN = auto()
    FINNISH = auto()
    TURKISH = auto()

    def get_bcp47_tag(self):
        """
        Returns the BCP47 tag for the language
        Usage:

        language = Languages.PORTUGUESE

        language.get_bcp47_tag()

        OR:

        Languages.PORTUGUESE.get_bcp47_tag()
        """
        return BCP47_TAGS[self.name]


def to_output(enum_class: Type[StrEnum]) -> str:
    formatted_enum = " ".join([f"{enum.value}" for enum in enum_class])
    return formatted_enum


def parse_languages(input_languages: Any):
    valid_languages: list[Languages] = []
    valid_string_languages = [language.value.lower() for language in Languages]

    for target_language in input_languages:
        if target_language.lower() in valid_string_languages:
            valid_languages.append(Languages[target_language.upper()])

    return valid_languages



# Tests for check_first_is_quote
from bragir.post_processing import (
    check_first_is_quote,
    check_last_is_quote,
    process_text,
    remove_all_newline_before_first_character,
    remove_newline_in_the_middle,
    remove_quoutes,
    remove_space,
    remove_space_as_first_character,
)


def test_check_first_is_quote_true():
    assert not check_first_is_quote("Hello")


def test_check_first_is_quote_false():
    assert check_first_is_quote('"Hello')


def test_check_first_is_quote_with_space():
    assert not check_first_is_quote(" No quotes")


# Tests for check_last_is_quote
def test_check_last_is_quote_true():
    assert not check_last_is_quote("Hello")


def test_check_last_is_quote_false():
    assert check_last_is_quote('Hello"')


def test_check_last_is_quote_with_space():
    assert not check_last_is_quote("No quotes ")


# Tests for remove_quotes
def test_remove_quotes_both_sides():
    assert remove_quoutes('"Hello"') == "Hello"


def test_remove_quotes_only_start():
    assert remove_quoutes('"Already processed" text') == 'Already processed" text'


def test_remove_quotes_no_quotes():
    assert remove_quoutes("Hello") == "Hello"


# Tests for remove_space
def test_remove_space_beginning():
    assert remove_space(" Hello", 0) == "Hello"


def test_remove_space_end():
    assert remove_space("Hello ", 5) == "Hello"


# Tests for remove_space_as_first_character
def test_remove_space_as_first_character_with_space():
    assert remove_space_as_first_character(" Hello") == "Hello"


def test_remove_space_as_first_character_no_space():
    assert remove_space_as_first_character("Hello") == "Hello"


def test_remove_space_as_first_character_with_multiple_words():
    assert remove_space_as_first_character(" Hello World") == "Hello World"


# Tests for remove_all_newline_before_first_character
def test_remove_all_newline_before_first_character_leading_newlines():
    assert remove_all_newline_before_first_character("\n\nHello") == "Hello"


def test_remove_all_newline_before_first_character_no_newlines():
    assert remove_all_newline_before_first_character("Hello") == "Hello"


def test_remove_all_newline_before_first_character_mixed_newlines():
    assert (
        remove_all_newline_before_first_character("\n\nHello\nWorld") == "Hello\nWorld"
    )


# Tests for remove_newline_in_the_middle
def test_remove_newline_in_the_middle_simple():
    assert remove_newline_in_the_middle("Hello\nWorld") == "Hello World"


def test_remove_newline_in_the_middle_complex():
    assert remove_newline_in_the_middle("\nStart\nMiddle\nEnd") == "\nStart Middle End"


# Tests for process_text
def test_process_text_example_1():
    text1 = "come si traduce in codice reale. E cosa \n\nquesto effettivamente"
    expected1 = "come si traduce in codice reale. E cosa questo effettivamente"
    assert process_text(text1) == expected1


def test_process_text_example_2():
    text2 = "\n\nUn database, puoi memorizzarli come file, puoi fare molte cose con\n"
    expected2 = "Un database, puoi memorizzarli come file, puoi fare molte cose con\n"
    assert process_text(text2) == expected2


def test_process_text_example_3():
    text3 = '"Sicuramente usa un database. Ma qui, lo sto facendo così."'
    expected3 = "Sicuramente usa un database. Ma qui, lo sto facendo così."
    assert process_text(text3) == expected3



import asyncio
from typing import Any

from openai import AsyncOpenAI, OpenAI
from pydantic import BaseModel

from bragir.config import get_config
from bragir.srt.srt_part import SRTPart
from bragir.timer import timing_decorator
from bragir.tracing.logger import logger


def split_by_breakpoints(
    collection: list[Any], breakpoints: list[int]
) -> list[list[Any]]:
    if len(collection) == 0 or len(collection) == 1:
        return collection

    result: list[Any] = []
    start_index = 0

    for breakpoint in breakpoints:
        result.append(collection[start_index:breakpoint])
        start_index = breakpoint

    # Append the remaining elements after the last breakpoint
    result.append(collection[start_index:])

    for res in result:
        if res == []:
            result.remove(res)

    return result


def translate(translator: OpenAI, part: SRTPart, language: str) -> str:
    config = get_config()

    if config is None:
        raise Exception("Config not found")

    #    model = config.client.model

    completion = translator.beta.chat.completions.parse(
        model="gpt-4o-2024-08-06",
        messages=[
            {
                "role": "system",
                "content": f"""
                You are a translation machine that only translates text to **{language}**.

                Translate only into {language} without altering any other language.

                Index is the following: {part.index}
            """,
            },
            {"role": "user", "content": part.content},
        ],
        response_format=Part,
    )

    return completion.choices[0].message.content or ""


async def async_translate(translator: AsyncOpenAI, part: SRTPart, language: str) -> str:
    config = get_config()

    if config is None:
        raise Exception("Config not found")

    #    model = config.client.model

    completion = await translator.beta.chat.completions.parse(
        model="gpt-4o-2024-08-06",
        messages=[
            {
                "role": "system",
                "content": f"""
                You are a translation machine that only translates text to **{language}**.

                Translate only into {language} without altering any other language.

                Index is the following: {part.index}

                Only return me the translated text.
            """,
            },
            {"role": "user", "content": part.content},
        ],
        response_format=Part,
    )

    translated_part = completion.choices[0].message.parsed

    if translated_part is None:
        raise Exception("Translated part is None")

    return translated_part.content


async def translate_srt_part(
    translator: AsyncOpenAI,
    part: SRTPart,
    language: str,
) -> SRTPart:
    config = get_config()

    if config is None:
        raise Exception("Config not found")

    translation = await async_translate(translator, part, language)

    part.translation = translation

    return part


@timing_decorator
def translate_srt_parts(
    translator: OpenAI, parts: list[SRTPart], language: str
) -> list[SRTPart]:
    for i, part in enumerate(parts):
        part.translation = translate(translator, part, language)
        logger.info(f"Translated {i+1}/{len(parts)}")

    return parts


async def async_translate_srt_parts(
    translator: AsyncOpenAI, srt_parts: list[SRTPart], language: str
) -> list[SRTPart]:
    tasks = [
        translate_srt_part(translator=translator, part=part, language=language)
        for part in srt_parts
    ]
    translated_parts = await asyncio.gather(*tasks)

    return translated_parts


class Part(BaseModel):
    index: int
    content: str



def read_file(file_path: str):
    with open(file=file_path, mode="r") as file:
        file_content = file.read()
    return file_content


def create_file(file_path: str, content: str):
    with open(file=file_path, mode="w", encoding="utf-8") as fileIO:
        fileIO.write(content)



import os
from configparser import ConfigParser
from enum import StrEnum, auto
from pathlib import Path
from typing import Any

import click
from pydantic import BaseModel

from bragir.constants.ai import POSSIBLE_MODELS
from bragir.tracing.logger import logger

CONFIG_NAME = "config.ini"
HOME_USER_PATH = Path.home()
BRAGIR_DIRECTORY = HOME_USER_PATH / ".bragir"
CONFIG_FILE_PATH = BRAGIR_DIRECTORY / CONFIG_NAME


BASE_CONFIG = {
    "audio": {
        "min_silence_len": "1000",
        "silence_thresh": "-40",
        "keep_silence": "True",
    },
    "logging": {
        "level": "info",
    },
    "client": {
        "openai_api_key": "YOUR_API_KEY",
        "model": POSSIBLE_MODELS[0],
    },
}


class LoggingLevel(StrEnum):
    DEBUG = auto()
    INFO = auto()
    WARNING = auto()
    ERROR = auto()
    CRITICAL = auto()


class AudioConfig(BaseModel):
    min_silence_len: int
    silence_thresh: int
    keep_silence: bool


class LoggingConfig(BaseModel):
    level: LoggingLevel


class ClientConfig(BaseModel):
    openai_api_key: str
    model: str


class Config(BaseModel):
    audio: AudioConfig
    logging: LoggingConfig
    client: ClientConfig


def create_config_file(file_path: Path = CONFIG_FILE_PATH) -> None:
    logger.info(f"Creating config file at: {file_path}")

    # Ensure bragir directory exists
    os.makedirs(BRAGIR_DIRECTORY, exist_ok=True)

    config_parser = ConfigParser()
    config_parser.read_dict(BASE_CONFIG)

    with open(file_path, "w") as file:
        config_parser.write(file)


def get_config_parser(file_path: Path) -> ConfigParser:
    config_parser = ConfigParser()
    config_parser.read(file_path)
    return config_parser


def find_section(
    key: str,
    config_parser: ConfigParser = get_config_parser(file_path=CONFIG_FILE_PATH),
) -> str | None:
    sections = config_parser.sections()

    for section in sections:
        if config_parser.has_option(section, key):
            return section

    return None


def reset_config_file(
    config_parser: ConfigParser = ConfigParser(), target_path: Path = CONFIG_FILE_PATH
) -> None:
    logger.info(f"Resetting config file at: {target_path}")
    config_parser.read_dict(BASE_CONFIG)

    with open(target_path, "w") as file:
        config_parser.write(file)


def set_config(config: Config, file_path: Path = CONFIG_FILE_PATH) -> None:
    logger.info(f"Setting config file at {file_path}")
    config_parser = get_config_parser(file_path=file_path)

    for section, values in config.model_dump().items():
        if not config_parser.has_section(section):
            config_parser.add_section(section)

        for key, value in values.items():
            config_parser.set(section, key, str(value))

    with open(file_path, "w") as file:
        config_parser.write(file)


def set_config_value(key: str, value: str, file_path: Path = CONFIG_FILE_PATH) -> None:
    config_parser = get_config_parser(file_path=file_path)
    section = find_section(key, config_parser)

    if section is None:
        logger.error(f"Section for key {key} not found {file_path}")
        exit(1)

    config_parser.set(section, key, value)

    with open(file_path, "w") as file:
        config_parser.write(file)


def get_config(file_path: Path = CONFIG_FILE_PATH) -> Config | None:
    config_parser = get_config_parser(file_path=file_path)

    config_dict = {
        section: dict(config_parser[section]) for section in config_parser.sections()
    }

    if config_dict == {}:
        return None

    return Config.model_validate(config_dict)


def read_config(
    file_path: Path = CONFIG_FILE_PATH,
):
    config = get_config(file_path=file_path)

    if config is None:
        logger.error("Config file not found")
        exit(1)

    for section_name, section_value in config.model_dump().items():
        click.echo(click.style(f"[{section_name}]", fg="cyan", bold=True))

        for key, value in section_value.items():
            if key == "openai_api_key":
                click.echo(
                    f"  {click.style(key, fg='yellow')} = {click.style("*************", fg='green')}"
                )
            else:
                click.echo(
                    f"  {click.style(key, fg='yellow')} = {click.style(str(value), fg='green')}"
                )

        click.echo()


def update_dict(config: dict[str, Any], updates: dict[str, Any]) -> dict[str, Any]:
    for key in config.keys():
        if isinstance(config[key], dict):
            config[key] = update_dict(config[key], updates)
        else:
            if key in updates:
                config[key] = updates[key]

    return config



import click

from bragir import commands
from bragir.client import initate_async_client
from bragir.config import create_config_file, get_config
from bragir.tracing.logger import setup_logging, logger
from bragir.tracing.stratergies import InfoLoggerStrategy

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.version_option()
def cli(context: click.Context) -> None:
    """
    Bragir is an tool that can generate SRT files from videos and translate SRT files.

    In order to use Bragir, an OpenAI api_key needs to be passed as an option. Or alternativly,
    as an enviroment variable in the current session
    """

    setup_logging(InfoLoggerStrategy())

    config = get_config()

    if config is None and context.invoked_subcommand == "config":
        logger.info("Config file not found")
        create_config_file()
        exit(0)

    if config is None:
        logger.error("Config file not found")
        logger.info(
            "Run the following command to create a new config file: `bragir config create`"
        )
        exit(1)

    context.ensure_object(dict)
    if not context.invoked_subcommand == "config":
        api_key: str = config.client.openai_api_key

        # TODO: Fix so the check does not always run, for example when calling the config commands
        # client = initiate_client(api_key=api_key)
        client = initate_async_client(api_key=api_key)

        context.obj["client"] = client

    context.obj["config"] = config


cli.add_command(commands.transcribe)
cli.add_command(commands.translate)
cli.add_command(commands.config)



from functools import reduce
import re


def check_first_is_quote(text: str) -> bool:
    first_letter = text[0]

    return first_letter == '"'


def check_last_is_quote(text: str) -> bool:
    last_letter = text[-1]

    return last_letter == '"'


def remove_quoutes(text: str) -> str:
    if check_first_is_quote(text) and check_last_is_quote(text):
        return text[1:-1]

    if check_first_is_quote(text):
        return text[1:]

    if check_last_is_quote(text):
        return text[:-1]

    return text


def remove_space(text: str, index: int) -> str:
    return text[:index] + text[index + 1 :]


def remove_space_as_first_character(text: str) -> str:
    if text[0] == " ":
        return remove_space(text, 0)
    else:
        return text


def remove_all_newline_before_first_character(text: str) -> str:
    while text[0] == "\n":
        text = remove_space(text, 0)
    return text


def remove_newline_in_the_middle(text: str) -> str:
    first_char = re.search(r"\S", text)
    if first_char is None:
        return text
    first_charachter_index = first_char.start()
    last_charachter_index = len(text) - 1

    in_the_middle_text = text[first_charachter_index:last_charachter_index]

    replaced_text = in_the_middle_text.replace("\n\n", "").replace("\n", " ")

    return text[:first_charachter_index] + replaced_text + text[last_charachter_index]


def process_text(text: str) -> str:
    pipeline = [
        remove_quoutes,
        remove_space_as_first_character,
        remove_all_newline_before_first_character,
        remove_newline_in_the_middle,
    ]

    result = reduce(lambda acc, func: func(acc), pipeline, text)

    return result



from .transcription import (
    transcribe_audio as transcribe_audio,
    transcribe_file as transcribe_file,
)
from .translation import (
    translate_srt_part as translate_srt_part,
    translate as translate,
)
from .languages import Languages as Languages
from .files.file import File as File



from .translation import (
    translate_srt_parts as translate_srt_parts,
    translate_srt_part as translate_srt_part,
    async_translate as async_translate,
    translate as translate,
    async_translate_srt_parts as async_translate_srt_parts,
    split_by_breakpoints as split_by_breakpoints,
)



from bragir.languages import Languages, to_output

PROMPT_HELP = {
    "file": "Path to file",
    "api_key": "Enter valid openai api key",
    "directory": "Enter one directory that contains the files that is going to be translated",
    "language": f"Enter one or more languages {to_output(Languages)}",
    "file_path": "Path to a .srt file in your system",
    "output_path": "Path to where the file should be stored in your system",
}


ACTIONS = {
    "translating": "Translating to following language/languages: {lanugages}",
}



import os

from bragir.constants.files import BLACKLISTED_FILES


def get_target_path(path: str, output: str) -> str:
    if output and os.path.isdir(output):
        root, _ = os.path.splitext(path)
        return os.path.join(output, os.path.basename(root) + ".srt")

    if output and os.path.isfile(output):
        return output

    root, _ = os.path.splitext(path)
    return root + ".srt"


def get_files(path: str) -> list[str]:
    paths: list[str] = []

    if os.path.isfile(path):
        return [path]

    for root, _dirs, nested_files in os.walk(path):
        for nested_file in nested_files:
            if nested_file not in BLACKLISTED_FILES:
                paths.append(os.path.join(root, nested_file))

    return paths



from typing import get_args
from openai.types import ChatModel


MODEL_TOKEN_LIMITS = {
    "gpt-4o": 128000,
    "gpt-4o-2024-08-06": 128000,
    "gpt-4o-mini": 128000,
    "gpt-4o-mini-2024-07-18": 128000,
    "gpt-4o-realtime-preview": 128000,
    "gpt-4o-audio-preview": 128000,
    "o1-preview": 128000,
    "o1-preview-2024-09-12": 128000,
    "o1-mini": 128000,
    "o1-mini-2024-09-12": 128000,
    "gpt-4-turbo": 128000,
    "gpt-4": 8192,
    "gpt-3.5-turbo": 16385,
}


POSSIBLE_MODELS: list[str] = list(get_args(ChatModel))


def get_model_limit(model: str) -> int | None:
    if model not in POSSIBLE_MODELS:
        return None

    return MODEL_TOKEN_LIMITS.get(model, 128000)



import openai
from openai import AuthenticationError, OpenAI

from openai import AsyncOpenAI
from bragir.tracing.logger import logger


def initiate_client(api_key: str) -> OpenAI:
    logger.info("Initiating client")
    if not api_key:
        logger.error("Cannot initiate OpenAI client, please provide an API key.")
        exit(1)

    if not check_validity(api_key):
        logger.error("Invalid API key.")
        exit(1)

    return OpenAI(api_key=api_key)


def initate_async_client(api_key: str) -> AsyncOpenAI:
    logger.info("Initiating async client")
    if not api_key:
        logger.error("Cannot initiate OpenAI client, please provide an API key.")
        exit(1)

    if not check_validity(api_key):
        logger.error("Invalid API key.")
        exit(1)

    return AsyncOpenAI(api_key=api_key)


def check_validity(api_key: str) -> bool:
    logger.info("Checking validity of API key")
    try:
        openai.api_key = api_key
        openai.models.list()
    except AuthenticationError as _e:
        return False
    return True



from .transcription import (
    transcribe_audio as transcribe_audio,
    transcribe_file as transcribe_file,
    transcribe_audio_files as transcribe_audio_files,
    async_transcribe_audio as async_transcribe_audio,
    async_transcribe_file as async_transcribe_file,
    async_transcribe_audio_files as async_transcribe_audio_files,
)



import asyncio
from pathlib import Path

import click
from openai import AsyncOpenAI

from bragir.config import (
    CONFIG_FILE_PATH,
    Config,
    create_config_file,
    get_config,
    read_config,
    reset_config_file,
    set_config,
    update_dict,
)
from bragir.constants.ai import POSSIBLE_MODELS, get_model_limit
from bragir.file import (
    get_new_file_path,
    get_srt_parts,
    read_file,
)
from bragir.files import create_file
from bragir.languages import Languages, parse_languages
from bragir.messages import PROMPT_HELP
from bragir.path import get_files, get_target_path
from bragir.post_processing import process_text
from bragir.spinner import spinner
from bragir.srt.srt_part import SRTPart
from bragir.time import update_timestamps
from bragir.tracing.logger import logger
from bragir.transcription import async_transcribe_file
from bragir.translation import async_translate_srt_parts


@click.command(options_metavar="<options>")
@click.argument(
    "path",
    type=click.Path(exists=True, file_okay=True, dir_okay=True),
    metavar="<path>",
)
@click.argument(
    "output",
    type=click.Path(exists=False, file_okay=True, dir_okay=True, writable=True),
    required=False,
)
@spinner("transcribe")
@click.pass_context
def transcribe(context: click.Context, path: str, output: str) -> None:
    """
    The transcribe command generates an SRT file based on an .mp4 or .mp3 file.
    If output is not set, then it will take the file_path name and change the extension.
    """
    logger.info("Starting transcription")

    transcriber: AsyncOpenAI = context.obj["client"]

    file_paths = get_files(path)

    for file_path in file_paths:
        transcripts = asyncio.run(
            async_transcribe_file(transcriber=transcriber, path=file_path)
        )

        print(transcripts)

        videos_srts: list[tuple[int, list[SRTPart]]] = [
            (order, get_srt_parts(transcript))
            for order, transcript in enumerate(transcripts)
        ]

        sorted_videos = sorted(videos_srts)

        srt_parts: list[SRTPart] = update_timestamps(sorted_videos)

        contents = "".join([srt_part.raw_srt_format for srt_part in srt_parts])

        target_path = get_target_path(file_path, output)

        with open(target_path, "w", encoding="utf-8") as fileIO:
            logger.info(f"Creating {target_path} for video {path}")
            fileIO.write(contents)


@click.command(options_metavar="<options>")
@click.argument(
    "path",
    type=click.Path(dir_okay=True, exists=True),
    metavar="<path>",
)
@click.option(
    "--language",
    "-l",
    required=True,
    type=click.Choice([language.value for language in Languages], case_sensitive=False),
    multiple=True,
    help=PROMPT_HELP["language"],
)
@click.pass_context
@spinner("translate")
def translate(context: click.Context, path: str, language: str) -> None:
    """
    The translate command, translates either a single SRT file or files or directory of SRT files into the wanted language.
    """
    translator: AsyncOpenAI = context.obj["client"]

    config = get_config()

    if config is None:
        logger.error("Config file not found")
        exit(1)

    logger.info("Config file found")

    limit = get_model_limit(config.client.model)

    if limit is None:
        logger.error("Model limit not found")
        exit(1)

    logger.info(f"Model limit: {limit}")

    file_paths = get_files(path)
    num_of_file_paths = len(file_paths)

    if num_of_file_paths == 1:
        logger.info(f"Translating {file_paths[0]}")
    else:
        logger.info(f"Translating {num_of_file_paths} files in directory {path}")

    translate_to_languages: list[Languages] = parse_languages(language)
    logger.info(
        f"Translating to following language/languages: {' '.join([language.value for language in translate_to_languages])}"
    )

    for file_path in file_paths:
        file_content = read_file(file_path)

        srt_parts: list[SRTPart] = get_srt_parts(file_content)

        translated_parts = asyncio.run(
            async_translate_srt_parts(
                translator=translator, srt_parts=srt_parts, language=language
            )
        )


        for translated_part in translated_parts:
            translated_part.translation = process_text(translated_part.translation)

        translation: str = ""

        for traslated_part in translated_parts:
            translation += traslated_part.translated_raw_srt_format

        for language in translate_to_languages:
            new_file_path = get_new_file_path(file_path, language)
            create_file(new_file_path, translation)
            logger.info(f"Created file {new_file_path}")


@click.group(invoke_without_command=False)
def config() -> None:
    pass


@config.command()
@click.option(
    "--path",
    "-p",
    required=False,
    type=click.Path(exists=False, file_okay=True, dir_okay=False),
    help="Path to the config file",
)
def create(path: Path | None) -> None:
    """
    Creates a new config file.
    """

    target_path = Path(path) if path else Path(CONFIG_FILE_PATH)

    if target_path.exists():
        logger.info(f"Config file already exists at {target_path}")
        exit(1)

    create_config_file(file_path=target_path)


@config.command()
@click.option(
    "--path",
    "-p",
    required=False,
    type=click.Path(exists=True, file_okay=True, dir_okay=True),
    default=CONFIG_FILE_PATH,
    help="Path to the config file",
)
def show(path: Path) -> None:
    """
    Displays the current config file.
    """

    target_path = Path(path) if path else Path(CONFIG_FILE_PATH)

    logger.info(f"Reading config file at {target_path}")
    read_config(file_path=target_path)


@config.command()
@click.option(
    "--path",
    "-p",
    required=False,
    type=click.Path(exists=True, file_okay=True, dir_okay=True),
    default=CONFIG_FILE_PATH,
    help="Path to the config file",
)
def reset(path: Path) -> None:
    """
    Resets the current config file to default.
    """

    target_path = Path(path) if path else Path(CONFIG_FILE_PATH)

    reset_config_file(target_path=target_path)


@config.command()
@click.option(
    "--path",
    "-p",
    required=False,
    type=click.Path(exists=True, file_okay=True, dir_okay=True),
    default=CONFIG_FILE_PATH,
)
@click.option(
    "--model",
    "-m",
    required=False,
    type=click.Choice(POSSIBLE_MODELS, case_sensitive=False),
    help="OpenAI model",
)
@click.option(
    "--min_silence_len",
    "-msl",
    required=False,
    type=click.IntRange(0, 10000),
    help="Minimum silence length in milliseconds",
)
@click.option(
    "--silence_thresh",
    "-st",
    required=False,
    type=click.IntRange(-100, 0),
    help="Silence threshold in dB",
)
@click.option(
    "--keep_silence",
    "-ks",
    required=False,
    type=click.BOOL,
    help="Keep silence",
    default=False,
)
def update(path: Path, **kwargs: dict[str, str | int | bool | None] | None) -> None:
    """
    Updates the config file with the given values of the options.
    """

    target_path = Path(path) if path else Path(CONFIG_FILE_PATH)

    config = get_config(file_path=target_path)

    if config is None:
        logger.error("Config file not found")
        exit(1)

    updates = {key: value for key, value in kwargs.items() if value is not None}

    updated_dict = update_dict(config.model_dump(), updates)

    updated_config = Config(**updated_dict)

    set_config(file_path=target_path, config=updated_config)

    for key, value in updates.items():
        logger.info(f"Updated {key} to {value}")



# type: ignore
# source: https://medium.com/@rodney_ragan/writing-a-spinner-a-spinning-progression-indicator-in-python-6f82e3a3bee
import functools
import sys
import threading
from datetime import timedelta
from itertools import cycle
from time import sleep, time

from bragir.tracing.logger import logger


def spin(msg: str, start: float, frames, stop_spin: threading.Event):
    while not stop_spin.is_set():
        frame = next(frames)
        sec, fsec = divmod(round(100 * (time() - start)), 100)
        frame += " ({} : {}.{:02.0f})".format(msg, timedelta(seconds=sec), fsec)
        print("\r", frame, sep="", end="", flush=True)
        sleep(0.1)


def spinner(msg: str = "Elapsed Time"):
    def decorator(func):
        @functools.wraps(func)
        def wrapper_decrorator(*args, **kwargs):
            stop_spin = threading.Event()
            start = time()
            spin_thread = threading.Thread(
                target=spin,
                args=(
                    msg,
                    start,
                    cycle(
                        [
                            " [    ]:",
                            " [=   ]:",
                            " [==  ]:",
                            " [=== ]:",
                            " [ ===]:",
                            " [  ==]:",
                            " [   =]:",
                        ]
                    ),
                    stop_spin,
                ),
            )
            spin_thread.start()

            try:
                value = func(*args, **kwargs)
            except Exception as e:
                logger.error(e)
                sys.exit(1)
            finally:
                stop = time()
                if spin_thread:
                    stop_spin.set()
                    spin_thread.join()
                print("\n", flush=True)
                print("=" * 60)
                print("Elapsed Time: ")
                print("=" * 60)
                print(stop - start)
                print("=" * 60)
                print()
            return value

        return wrapper_decrorator

    return decorator
